@@ -32,10 +32,7 @@
 #include "ruby/internal/warning_push.h"
 
 /** @cond INTERNAL_MACRO */
-#if RBIMPL_COMPILER_SINCE(MSVC, 13, 10, 0)
-# define RBIMPL_HAVE___ASSUME
-
-#elif RBIMPL_COMPILER_SINCE(Intel, 13, 0, 0)
+#if defined(HAVE___ASSUME)
 # define RBIMPL_HAVE___ASSUME
 #endif
 /** @endcond */
