@@ -46,6 +46,7 @@ require 'prettyprint'
 #
 # To define a customized pretty printing function for your classes,
 # redefine method <code>#pretty_print(pp)</code> in the class.
+# Note that <code>require 'pp'</code> is needed before redefining <code>#pretty_print(pp)</code>.
 #
 # <code>#pretty_print</code> takes the +pp+ argument, which is an instance of the PP class.
 # The method uses #text, #breakable, #nest, #group and #pp to print the
@@ -61,6 +62,9 @@ require 'prettyprint'
 # Tanaka Akira <akr@fsij.org>
 
 class PP < PrettyPrint
+
+  VERSION = "0.5.0"
+
   # Returns the usable width for +out+.
   # As the width of +out+:
   # 1. If +out+ is assigned to a tty device, its width is used.
@@ -416,6 +420,26 @@ class Struct # :nodoc:
   end
 end
 
+class Data # :nodoc:
+  def pretty_print(q) # :nodoc:
+    q.group(1, sprintf("#<data %s", PP.mcall(self, Kernel, :class).name), '>') {
+      q.seplist(PP.mcall(self, Data, :members), lambda { q.text "," }) {|member|
+        q.breakable
+        q.text member.to_s
+        q.text '='
+        q.group(1) {
+          q.breakable ''
+          q.pp public_send(member)
+        }
+      }
+    }
+  end
+
+  def pretty_print_cycle(q) # :nodoc:
+    q.text sprintf("#<data %s:...>", PP.mcall(self, Kernel, :class).name)
+  end
+end if "3.2" <= RUBY_VERSION
+
 class Range # :nodoc:
   def pretty_print(q) # :nodoc:
     q.pp self.begin
@@ -609,10 +633,6 @@ end
 module Kernel
   # Returns a pretty printed object as a string.
   #
-  # In order to use this method you must first require the PP module:
-  #
-  #   require 'pp'
-  #
   # See the PP module for more information.
   def pretty_inspect
     PP.pp(self, ''.dup)
