@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.3.") or
-    raise "ruby lib version (3.3.9) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.3.10) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.3/riscv64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "3"
-  CONFIG["TEENY"] = "9"
-  CONFIG["PATCHLEVEL"] = "170"
-  CONFIG["INSTALL"] = '/local/pobj/ruby-3.3.9/bin/install -c'
+  CONFIG["TEENY"] = "10"
+  CONFIG["PATCHLEVEL"] = "183"
+  CONFIG["INSTALL"] = '/local/pobj/ruby-3.3.10/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)33"
@@ -160,9 +160,9 @@ module RbConfig
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/local/pobj/ruby-3.3.9/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/local/pobj/ruby-3.3.9/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/local/pobj/ruby-3.3.9/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/local/pobj/ruby-3.3.10/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/local/pobj/ruby-3.3.10/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/local/pobj/ruby-3.3.10/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -213,7 +213,7 @@ module RbConfig
   CONFIG["RUBY_BASE_NAME"] = "ruby"
   CONFIG["RUBY_PROGRAM_VERSION"] = "$(MAJOR).$(MINOR).$(TEENY)"
   CONFIG["RUBY_API_VERSION"] = "$(MAJOR).$(MINOR)"
-  CONFIG["HAVE_GIT"] = "yes"
+  CONFIG["HAVE_GIT"] = "no"
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
