@@ -65,6 +65,12 @@ class TestMinitestReporter < MetaMetaMetaTestCase
     @pt ||= Minitest::Result.from Minitest::Test.new(:woot)
   end
 
+  def passing_test_with_metadata
+    test = Minitest::Test.new(:woot)
+    test.metadata[:meta] = :data
+    @pt ||= Minitest::Result.from test
+  end
+
   def skip_test
     unless defined? @st then
       @st = Minitest::Test.new(:woot)
@@ -166,6 +172,29 @@ class TestMinitestReporter < MetaMetaMetaTestCase
     assert_equal 0, r.assertions
   end
 
+  def test_record_pass_with_metadata
+    reporter = self.r
+
+    def reporter.metadata
+      @metadata
+    end
+
+    def reporter.record result
+      super
+      @metadata = result.metadata if result.metadata?
+    end
+
+    r.record passing_test_with_metadata
+
+    exp = { :meta => :data }
+    assert_equal exp, reporter.metadata
+
+    assert_equal ".", io.string
+    assert_empty r.results
+    assert_equal 1, r.count
+    assert_equal 0, r.assertions
+  end
+
   def test_record_fail
     fail_test = self.fail_test
     r.record fail_test
