@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 
 module RBS
   module Types
@@ -27,6 +28,14 @@ module RBS
           enum_for :each_type
         end
       end
+
+      def map_type(&block)
+        if block
+          _ = self
+        else
+          enum_for(:map_type)
+        end
+      end
     end
 
     module Bases
@@ -64,7 +73,7 @@ module RBS
           when Types::Bases::Void
             'void'
           when Types::Bases::Any
-            'untyped'
+            raise
           when Types::Bases::Nil
             'nil'
           when Types::Bases::Top
@@ -81,11 +90,32 @@ module RBS
             raise "Unexpected base type: #{inspect}"
           end
         end
+
+        def has_self_type?
+          self.is_a?(Types::Bases::Self)
+        end
+
+        def has_classish_type?
+          self.is_a?(Bases::Instance) || self.is_a?(Bases::Class)
+        end
+
+        def with_nonreturn_void?
+          self.is_a?(Bases::Void)
+        end
       end
 
       class Bool < Base; end
       class Void < Base; end
-      class Any < Base; end
+      class Any < Base
+        def to_s(level=0)
+          @string || "untyped"
+        end
+
+        def todo!
+          @string = '__todo__'
+          self
+        end
+      end
       class Nil < Base; end
       class Top < Base; end
       class Bottom < Base; end
@@ -153,6 +183,18 @@ module RBS
       end
 
       include EmptyEachType
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
     end
 
     class ClassSingleton
@@ -193,6 +235,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
     end
 
     module Application
@@ -206,7 +260,7 @@ module RBS
       alias eql? ==
 
       def hash
-        self.class.hash ^ name.hash ^ args.hash
+        name.hash ^ args.hash
       end
 
       def free_variables(set = Set.new)
@@ -232,6 +286,25 @@ module RBS
           enum_for :each_type
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? do |type|
+          if type.is_a?(Bases::Void)
+            # `void` in immediate generics parameter is allowed
+            false
+          else
+            type.with_nonreturn_void?
+          end
+        end
+      end
     end
 
     class Interface
@@ -262,6 +335,18 @@ module RBS
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Interface.new(
+            name: name,
+            args: args.map {|type| yield type },
+            location: location
+          )
+        else
+          enum_for(:map_type)
+        end
+      end
     end
 
     class ClassInstance
@@ -292,46 +377,58 @@ module RBS
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          ClassInstance.new(
+            name: name,
+            args: args.map {|type| yield type },
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
     end
 
     class Alias
       attr_reader :location
-      attr_reader :name
 
-      def initialize(name:, location:)
+      include Application
+
+      def initialize(name:, args:, location:)
         @name = name
+        @args = args
         @location = location
       end
 
-      def ==(other)
-        other.is_a?(Alias) && other.name == name
-      end
-
-      alias eql? ==
-
-      def hash
-        self.class.hash ^ name.hash
-      end
-
-      include NoFreeVariables
-      include NoSubst
-
       def to_json(state = _ = nil)
-        { class: :alias, name: name, location: location }.to_json(state)
+        { class: :alias, name: name, args: args, location: location }.to_json(state)
       end
 
-      def to_s(level = 0)
-        name.to_s
+      def sub(s)
+        Alias.new(name: name, args: args.map {|ty| ty.sub(s) }, location: location)
       end
 
-      include EmptyEachType
-
-      def map_type_name
+      def map_type_name(&block)
         Alias.new(
           name: yield(name, location, self),
+          args: args.map {|arg| arg.map_type_name(&block) },
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Alias.new(
+            name: name,
+            args: args.map {|type| yield type },
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
     end
 
     class Tuple
@@ -392,6 +489,29 @@ module RBS
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Tuple.new(
+            types: types.map {|type| yield type },
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Record
@@ -434,7 +554,7 @@ module RBS
         return "{ }" if self.fields.empty?
 
         fields = self.fields.map do |key, type|
-          if key.is_a?(Symbol) && key.match?(/\A[A-Za-z_][A-Za-z_]*\z/) && !key.match?(Parser::KEYWORDS_RE)
+          if key.is_a?(Symbol) && key.match?(/\A[A-Za-z_][A-Za-z_0-9]*\z/)
             "#{key}: #{type}"
           else
             "#{key.inspect} => #{type}"
@@ -457,6 +577,29 @@ module RBS
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Record.new(
+            fields: fields.transform_values {|type| yield type },
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Optional
@@ -497,6 +640,8 @@ module RBS
           when Symbol
             return "#{type.to_s(1)} ?"
           end
+        when RBS::Types::Proc
+          return "(#{type.to_s(1)})?"
         end
 
         "#{type.to_s(1)}?"
@@ -516,6 +661,29 @@ module RBS
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Optional.new(
+            type: yield(type),
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Union
@@ -584,6 +752,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Intersection
@@ -653,6 +833,18 @@ module RBS
           location: location
         )
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        each_type.any? {|type| type.with_nonreturn_void? }
+      end
     end
 
     class Function
@@ -691,10 +883,10 @@ module RBS
 
         def to_s
           if name
-            if /\A#{Parser::KEYWORDS_RE}\z/.match?(name)
-              "#{type} `#{name}`"
-            else
+            if name.match?(/\A[a-zA-Z0-9_]+\z/)
               "#{type} #{name}"
+            else
+              "#{type} `#{name}`"
             end
           else
             "#{type}"
@@ -779,12 +971,12 @@ module RBS
       def map_type(&block)
         if block
           Function.new(
-            required_positionals: required_positionals.map {|param| param.map_type(&block) },
-            optional_positionals: optional_positionals.map {|param| param.map_type(&block) },
+            required_positionals: amap(required_positionals) {|param| param.map_type(&block) },
+            optional_positionals: amap(optional_positionals) {|param| param.map_type(&block) },
             rest_positionals: rest_positionals&.yield_self {|param| param.map_type(&block) },
-            trailing_positionals: trailing_positionals.map {|param| param.map_type(&block) },
-            required_keywords: required_keywords.transform_values {|param| param.map_type(&block) },
-            optional_keywords: optional_keywords.transform_values {|param| param.map_type(&block) },
+            trailing_positionals: amap(trailing_positionals) {|param| param.map_type(&block) },
+            required_keywords: hmapv(required_keywords) {|param| param.map_type(&block) },
+            optional_keywords: hmapv(optional_keywords) {|param| param.map_type(&block) },
             rest_keywords: rest_keywords&.yield_self {|param| param.map_type(&block) },
             return_type: yield(return_type)
           )
@@ -793,6 +985,22 @@ module RBS
         end
       end
 
+      def amap(array, &block)
+        if array.empty?
+          _ = array
+        else
+          array.map(&block)
+        end
+      end
+
+      def hmapv(hash, &block)
+        if hash.empty?
+          _ = hash
+        else
+          hash.transform_values(&block)
+        end
+      end
+
       def map_type_name(&block)
         map_type do |type|
           type.map_type_name(&block)
@@ -951,69 +1159,110 @@ module RBS
           false
         end
       end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        if each_param.any? {|param| param.type.with_nonreturn_void? }
+          true
+        else
+          if return_type.is_a?(Bases::Void)
+            false
+          else
+            return_type.with_nonreturn_void?
+          end
+        end
+      end
     end
 
     class Block
       attr_reader :type
       attr_reader :required
+      attr_reader :self_type
 
-      def initialize(type:, required:)
+      def initialize(type:, required:, self_type: nil)
         @type = type
         @required = required ? true : false
+        @self_type = self_type
       end
 
       def ==(other)
         other.is_a?(Block) &&
           other.type == type &&
-          other.required == required
+          other.required == required &&
+          other.self_type == self_type
       end
 
       def to_json(state = _ = nil)
         {
           type: type,
-          required: required
+          required: required,
+          self_type: self_type
         }.to_json(state)
       end
 
       def sub(s)
         self.class.new(
           type: type.sub(s),
-          required: required
+          required: required,
+          self_type: self_type&.sub(s)
         )
       end
 
       def map_type(&block)
         Block.new(
           required: required,
-          type: type.map_type(&block)
+          type: type.map_type(&block),
+          self_type: self_type ? yield(self_type) : nil
         )
       end
     end
 
+    module SelfTypeBindingHelper
+      module_function
+
+      def self_type_binding_to_s(t)
+        if t
+          "[self: #{t}] "
+        else
+          ""
+        end
+      end
+    end
+
     class Proc
       attr_reader :type
       attr_reader :block
+      attr_reader :self_type
       attr_reader :location
 
-      def initialize(location:, type:, block:)
+      def initialize(location:, type:, block:, self_type: nil)
         @type = type
         @block = block
         @location = location
+        @self_type = self_type
       end
 
       def ==(other)
-        other.is_a?(Proc) && other.type == type && other.block == block
+        other.is_a?(Proc) && other.type == type && other.block == block && other.self_type == self_type
       end
 
       alias eql? ==
 
       def hash
-        self.class.hash ^ type.hash ^ block.hash
+        self.class.hash ^ type.hash ^ block.hash ^ self_type.hash
       end
 
       def free_variables(set = Set[])
         type.free_variables(set)
         block&.type&.free_variables(set)
+        self_type&.free_variables(set)
         set
       end
 
@@ -1022,24 +1271,33 @@ module RBS
           class: :proc,
           type: type,
           block: block,
-          location: location
+          location: location,
+          self_type: self_type
         }.to_json(state)
       end
 
       def sub(s)
-        self.class.new(type: type.sub(s), block: block&.sub(s), location: location)
+        self.class.new(
+          type: type.sub(s),
+          block: block&.sub(s),
+          self_type: self_type&.sub(s),
+          location: location
+        )
       end
 
       def to_s(level = 0)
+        self_binding = SelfTypeBindingHelper.self_type_binding_to_s(self_type)
+        block_self_binding = SelfTypeBindingHelper.self_type_binding_to_s(block&.self_type)
+
         case
         when b = block
           if b.required
-            "^(#{type.param_to_s}) { (#{b.type.param_to_s}) -> #{b.type.return_to_s} } -> #{type.return_to_s}"
+            "^(#{type.param_to_s}) #{self_binding}{ (#{b.type.param_to_s}) #{block_self_binding}-> #{b.type.return_to_s} } -> #{type.return_to_s}"
           else
-            "^(#{type.param_to_s}) ?{ (#{b.type.param_to_s}) -> #{b.type.return_to_s} } -> #{type.return_to_s}"
+            "^(#{type.param_to_s}) #{self_binding}?{ (#{b.type.param_to_s}) #{block_self_binding}-> #{b.type.return_to_s} } -> #{type.return_to_s}"
           end
         else
-          "^(#{type.param_to_s}) -> #{type.return_to_s}"
+          "^(#{type.param_to_s}) #{self_binding}-> #{type.return_to_s}"
         end
       end
 
@@ -1047,6 +1305,9 @@ module RBS
         if block
           type.each_type(&block)
           self.block&.type&.each_type(&block)
+          if self_type = self.block&.self_type
+            yield self_type
+          end
         else
           enum_for :each_type
         end
@@ -1056,9 +1317,43 @@ module RBS
         Proc.new(
           type: type.map_type_name(&block),
           block: self.block&.map_type {|type| type.map_type_name(&block) },
+          self_type: self_type&.map_type_name(&block),
           location: location
         )
       end
+
+      def map_type(&block)
+        if block
+          Proc.new(
+            type: type.map_type(&block),
+            block: self.block&.map_type(&block),
+            self_type: self_type ? yield(self_type) : nil,
+            location: location
+          )
+        else
+          enum_for :map_type
+        end
+      end
+
+      def has_self_type?
+        each_type.any? {|type| type.has_self_type? }
+      end
+
+      def has_classish_type?
+        each_type.any? {|type| type.has_classish_type? }
+      end
+
+      def with_nonreturn_void?
+        if type.with_nonreturn_void?
+          true
+        else
+          if block = block()
+            block.type.with_nonreturn_void? || block.self_type&.with_nonreturn_void? || false
+          else
+            false
+          end
+        end
+      end
     end
 
     class Literal
@@ -1092,6 +1387,47 @@ module RBS
       def to_s(level = 0)
         literal.inspect
       end
+
+      def has_self_type?
+        false
+      end
+
+      def has_classish_type?
+        false
+      end
+
+      def with_nonreturn_void?
+        false
+      end
+
+      TABLE = {
+        "\\a" => "\a",
+        "\\b" => "\b",
+        "\\e" => "\033",
+        "\\f" => "\f",
+        "\\n" => "\n",
+        "\\r" => "\r",
+        "\\s" => " ",
+        "\\t" => "\t",
+        "\\v" => "\v",
+        "\\\"" => "\"",
+        "\\\'" => "'",
+        "\\\\" => "\\",
+        "\\" => ""
+      }
+
+      def self.unescape_string(string, is_double_quote)
+        if is_double_quote
+          string.gsub!(/\\([0-9]{1,3})/) { ($1 || "").to_i(8).chr }
+          string.gsub!(/\\x([0-9a-f]{1,2})/) { ($1 || "").to_i(16).chr }
+          string.gsub!(/\\u([0-9a-fA-F]{4})/) { ($1 || "").to_i(16).chr(Encoding::UTF_8) }
+          string.gsub!(/\\[abefnrstv"'\\]?/, TABLE)
+          string
+        else
+          string.gsub!(/\\['\\]/, TABLE)
+          string
+        end
+      end
     end
   end
 end
