@@ -624,7 +624,7 @@ module TypeProf
     end
 
     def self.file_load(path, ep, env, scratch, &ctn)
-      iseq = ISeq.compile(path)
+      iseq, = ISeq.compile(path)
       callee_ep, callee_env = TypeProf.starting_state(iseq)
       scratch.merge_env(callee_ep, callee_env)
 
@@ -793,27 +793,27 @@ module TypeProf
 
       Import.import_builtin(scratch)
 
-      Type::Builtin[:vmcore]     = scratch.new_class(klass_obj, :VMCore, [], klass_obj, nil)
-      Type::Builtin[:int]        = scratch.get_constant(klass_obj, :Integer)
-      Type::Builtin[:float]      = scratch.get_constant(klass_obj, :Float)
-      Type::Builtin[:rational]   = scratch.get_constant(klass_obj, :Rational)
-      Type::Builtin[:complex]    = scratch.get_constant(klass_obj, :Complex)
-      Type::Builtin[:sym]        = scratch.get_constant(klass_obj, :Symbol)
-      Type::Builtin[:str]        = scratch.get_constant(klass_obj, :String)
-      Type::Builtin[:struct]     = scratch.get_constant(klass_obj, :Struct)
-      Type::Builtin[:ary]        = scratch.get_constant(klass_obj, :Array)
-      Type::Builtin[:hash]       = scratch.get_constant(klass_obj, :Hash)
-      Type::Builtin[:io]         = scratch.get_constant(klass_obj, :IO)
-      Type::Builtin[:proc]       = scratch.get_constant(klass_obj, :Proc)
-      Type::Builtin[:range]      = scratch.get_constant(klass_obj, :Range)
-      Type::Builtin[:regexp]     = scratch.get_constant(klass_obj, :Regexp)
-      Type::Builtin[:matchdata]  = scratch.get_constant(klass_obj, :MatchData)
-      Type::Builtin[:class]      = scratch.get_constant(klass_obj, :Class)
-      Type::Builtin[:module]     = scratch.get_constant(klass_obj, :Module)
-      Type::Builtin[:exc]        = scratch.get_constant(klass_obj, :Exception)
-      Type::Builtin[:encoding]   = scratch.get_constant(klass_obj, :Encoding)
-      Type::Builtin[:enumerator] = scratch.get_constant(klass_obj, :Enumerator)
-      Type::Builtin[:kernel]     = scratch.get_constant(klass_obj, :Kernel)
+      Type::Builtin[:vmcore]     , = scratch.new_class(klass_obj, :VMCore, [], klass_obj, nil)
+      Type::Builtin[:int]        , = scratch.get_constant(klass_obj, :Integer)
+      Type::Builtin[:float]      , = scratch.get_constant(klass_obj, :Float)
+      Type::Builtin[:rational]   , = scratch.get_constant(klass_obj, :Rational)
+      Type::Builtin[:complex]    , = scratch.get_constant(klass_obj, :Complex)
+      Type::Builtin[:sym]        , = scratch.get_constant(klass_obj, :Symbol)
+      Type::Builtin[:str]        , = scratch.get_constant(klass_obj, :String)
+      Type::Builtin[:struct]     , = scratch.get_constant(klass_obj, :Struct)
+      Type::Builtin[:ary]        , = scratch.get_constant(klass_obj, :Array)
+      Type::Builtin[:hash]       , = scratch.get_constant(klass_obj, :Hash)
+      Type::Builtin[:io]         , = scratch.get_constant(klass_obj, :IO)
+      Type::Builtin[:proc]       , = scratch.get_constant(klass_obj, :Proc)
+      Type::Builtin[:range]      , = scratch.get_constant(klass_obj, :Range)
+      Type::Builtin[:regexp]     , = scratch.get_constant(klass_obj, :Regexp)
+      Type::Builtin[:matchdata]  , = scratch.get_constant(klass_obj, :MatchData)
+      Type::Builtin[:class]      , = scratch.get_constant(klass_obj, :Class)
+      Type::Builtin[:module]     , = scratch.get_constant(klass_obj, :Module)
+      Type::Builtin[:exc]        , = scratch.get_constant(klass_obj, :Exception)
+      Type::Builtin[:encoding]   , = scratch.get_constant(klass_obj, :Encoding)
+      Type::Builtin[:enumerator] , = scratch.get_constant(klass_obj, :Enumerator)
+      Type::Builtin[:kernel]     , = scratch.get_constant(klass_obj, :Kernel)
 
       klass_vmcore = Type::Builtin[:vmcore]
       klass_ary    = Type::Builtin[:ary]
@@ -881,7 +881,7 @@ module TypeProf
       # ENV: Hash[String, String]
       str_ty = Type::Instance.new(Type::Builtin[:str])
       env_ty = Type.gen_hash {|h| h[str_ty] = Type.optional(str_ty) }
-      scratch.add_constant(klass_obj, :ENV, env_ty, false)
+      scratch.add_constant(klass_obj, :ENV, env_ty, nil)
 
       scratch.search_method(Type::Builtin[:kernel], false, :sprintf) do |mdefs,|
         mdefs.each do |mdef|
